@@ -65,10 +65,11 @@ extern void hp_init __((struct hostent *hp));
 extern void hp_setalist __((struct hostent *hp));
 extern char **hp_getaddr __((void));
 extern char **hp_nextaddr __((void));
+extern void hp_addr_randomize __((struct hostent *hp));
 #endif
 
 /* killprev.c */
-extern void killprevious __((int sig, char *pidfile));
+extern int killprevious __((int sig, char *pidfile));
 
 /* linebuffer.c */
 extern void initline __((long blksize));
@@ -97,7 +98,7 @@ extern u_int ranny __((u_int m));
 
 /* rfc822date.c */
 extern char *rfc822tz __((struct tm *ts, int prettyname));
-extern char *rfc822date __((long *unixtimep));
+extern char *rfc822date __((time_t *timep));
 
 /* rfc822scan.c */
 extern int    hdr_status __((u_char *cp, u_char *lbuf, int n));
@@ -120,7 +121,7 @@ extern int ci2strncmp __((char *a, char *b, int n));
 extern int strmatch __((unsigned char *pattern, unsigned char *term));
 
 /* symbol.c */
-extern u_int symbol __((u_char *s));
+extern spkey_t symbol __((u_char *s));
 extern char *pname __((u_int id));
 #ifdef MALLOC_TRACE
 extern int icpname __((struct spblk *spl));
