@@ -50,8 +50,8 @@ extern struct headerinfo *find_header();
 extern struct headerinfo *find_header();
 extern struct headerinfo envelope_hdrs[];
 extern struct headerinfo mandatory_hdrs[];
-extern struct token	*scan822();
-extern u_char	*rfc822date();
+extern struct token * scan822 __((u_char *cpp, long n, u_char c1, u_char c2,
+				  int allowcomments, struct token *tlistp));
 extern union misc parse822();
 extern void	dumpHeader();
 extern void	hdr_print();
