@@ -2,6 +2,11 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  */
+/*
+ *	Lots of modifications (new guts, more or less..) by
+ *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ */
+
 
 /*
  * ZMailer router, main and miscellany routines.
@@ -22,6 +27,7 @@
 #include "splay.h"
 
 #include "prototypes.h"
+#include "libz.h"
 
 extern int loginit();/* should be SIGNAL_TYPE but we're playing games */
 extern int nobody;
@@ -34,11 +40,10 @@ extern struct passwd *getpwuid();
 
 extern struct shCmd fnctns[];
 extern time_t time();
-extern void getnobody(), prversion();
+extern void prversion();
 extern void tfree();
 extern void trapexit();
 extern void detach();
-extern void killprevious();
 extern void zshinit();
 extern void setfreefd();
 extern void init_header();
@@ -64,6 +69,7 @@ int	deferit;
 int	origoptind;
 int	savefile = 0;
 char	*zshopts = "-O";
+int	nosyslog = 1;
 
 int
 main(argc, argv)
@@ -90,7 +96,7 @@ main(argc, argv)
 	nrouters = 1;
 
 
-	while ((c = getopt(argc, argv, "m:n:dikf:o:t:L:P:V")) != EOF)
+	while ((c = getopt(argc, argv, "m:n:dikf:o:t:L:P:SV")) != EOF)
 		switch (c) {
 		case 'd':	/* become a daemon */
 			daemonflg = 1;
@@ -138,6 +144,10 @@ main(argc, argv)
 		case 'k':	/* kill the previous daemon upon startup */
 			killflg = 1;
 			break;
+		case 'S':	/* Logging also always to SYSLOG,
+				   not only at serious stuff */
+			nosyslog = 0;
+			break;
 		case 'L':	/* override default log file */
 			log = optarg;
 			break;
@@ -227,43 +237,47 @@ main(argc, argv)
 
 	if (daemonflg) {
 		if (chdir(postoffice) < 0 || chdir(ROUTERDIR) < 0)
-			fprintf(stderr, "%s: cannot chdir.\n", progname);
+		  fprintf(stderr, "%s: cannot chdir.\n", progname);
 		/* XX: check if another daemon is running already */
 		if (offout < ftell(stdout) || offerr < ftell(stderr)) {
-			fprintf(stderr, "%d %d %d %d\n",
-				offout, ftell(stdout), offerr, ftell(stderr));
-			fprintf(stderr, "%s: daemon not started.\n", progname);
-			die(1, "errors during startup");
+		  fprintf(stderr, "%d %d %d %d\n",
+			  offout, ftell(stdout), offerr, ftell(stderr));
+		  fprintf(stderr, "%s: daemon not started.\n", progname);
+		  die(1, "errors during startup");
 		}
 		if (tac == 0)		/* leave worldy matters behind */
-			detach();
+		  detach();
 		printf("%s: router daemon (%s)\n\tstarted at %s\n",
 		       progname, Version, (char *)rfc822date(&now));
 		if (killflg)
-			killprevious(-SIGTERM, pidfile);
+		  if (killprevious(-SIGTERM, pidfile) != 0) {
+		    /* Indicates failure at pidfile creation! */
+		    fprintf(stderr,"router can't create pidfile ?? Disk full ??\n");
+		    exit(2);
+		  }
 		if (nrouters > 1) {
-			int pgrp, ppid = getpid();
-#ifdef	USE_BSDSETPGRP /* We assume getpgrp() and setpgrp() calling
-			  conventions match in the machine.. */
-			pgrp = getpgrp(ppid);	/* BSD  -style */
+		  int pgrp, ppid = getpid();
+#ifdef	USE_BSDSETPGRP		/* We assume getpgrp() and setpgrp() calling
+				   conventions match in the machine.. */
+		  pgrp = getpgrp(ppid);	/* BSD  -style */
 #else
-			pgrp = getpgrp();	/* SysV/POSIX -style */
+		  pgrp = getpgrp(); /* SysV/POSIX -style */
 #endif
-			if (pgrp != ppid) {
-				fprintf(stderr, "process group %d != pid %d\n",
-					pgrp, ppid);
-				fprintf(stderr, "%s: daemon not started.\n", progname);
-				die(1, "capability error during startup");
-			}
-			for (router_id = 2; router_id <= nrouters; ++router_id){
-				if (fork() <= 0)	/* child or error */
-					break;
-				/* parent */
-			}
-			if (router_id == nrouters + 1)
-				router_id = 1;
-			printf("%s: number %d started as pid %d\n",
-					progname, router_id, getpid());
+		  if (pgrp != ppid) {
+		    fprintf(stderr, "process group %d != pid %d\n",
+			    pgrp, ppid);
+		    fprintf(stderr, "%s: daemon not started.\n", progname);
+		    die(1, "capability error during startup");
+		  }
+		  for (router_id = 2; router_id <= nrouters; ++router_id){
+		    if (fork() <= 0) /* child or error */
+		      break;
+		    /* parent */
+		  }
+		  if (router_id == nrouters + 1)
+		    router_id = 1;
+		  printf("%s: number %d started as pid %d\n",
+			 progname, router_id, getpid());
 		}
 #ifdef	LOG_PID
 #ifdef	LOG_MAIL
@@ -274,38 +288,39 @@ main(argc, argv)
 #endif	/* LOG_PID */
 	}
 	if (c < argc) {
-		savefile = 1;
-		/*
-		 * we need to use a local variable (c) because optind is global
-		 * and can (and will) be modified by the funcall()'s we do.
-		 */
-		do {
-			av[0] = "process";
-			av[1] = argv[c];
-			av[2] = NULL;
+	  savefile = 1;
+	  /*
+	   * we need to use a local variable (c) because optind is global
+	   * and can (and will) be modified by the funcall()'s we do.
+	   */
+	  do {
+	    av[0] = "process";
+	    av[1] = argv[c];
+	    av[2] = NULL;
 #ifdef	XMEM
 write(30, "\n", 1);
 #endif	/* XMEM */
-			s_apply(2, &av[0]);
-		} while (++c < argc);
+	    s_apply(2, &av[0]);
+	  } while (++c < argc);
 	} else if (daemonflg) {
-		av[0] = "daemon";
-		av[1] = NULL;
-		run_daemon(1, &av[0]);
-		/* NOTREACHED */
+	  av[0] = "daemon";
+	  av[1] = NULL;
+	  run_daemon(1, &av[0]);
+	  /* NOTREACHED */
 	} else if (interactiveflg) {
 #ifdef	MALLOC_TRACE
-		zshtoplevel((char *)NULL);
+	  zshtoplevel((char *)NULL);
 #else	/* !MALLOC_TRACE */
-		trapexit(zshtoplevel((char *)NULL));
+	  trapexit(zshtoplevel((char *)NULL));
 #endif	/* MALLOC_TRACE */
-		/* NOTREACHED */
+	  /* NOTREACHED */
 	}
 #ifdef	MALLOC_TRACE
-	dbfree(); zshfree();
+	dbfree();
+	zshfree();
 #endif	/* MALLOC_TRACE */
 	if (mustexit)
-		die(0, "signal");
+	  die(0, "signal");
 #ifdef	MALLOC_TRACE
 	die(0, "malloc trace");
 #endif	/* MALLOC_TRACE */
@@ -371,7 +386,7 @@ initialize(configfile, argc, argv)
 
 	/* trusted users */
 	for (cpp = default_trusted; cpp != NULL && *cpp != NULL; ++cpp)
-		sp_install(symbol((u_char *)*cpp), (char *)NULL, 0, spt_goodguys);
+		sp_install(symbol((u_char *)*cpp), NULL, 0, spt_goodguys);
 
 	if (files_group != NULL) {
 		if ((grp = getgrnam(files_group)) == NULL)
@@ -563,7 +578,8 @@ logit(file, id, from, to)
 		*(from+MAXSAFESIZE-baselen) = '\0';
 		printf("%s: file: %s %s...\n", id, file, from);
 #ifdef	LOG_INFO
-		syslog(LOG_INFO, "%s: file: %s %s...", id, file, from);
+		if (!nosyslog)
+		  syslog(LOG_INFO, "%s: file: %s %s...", id, file, from);
 #endif	/* LOG_INFO */
 		from += MAXSAFESIZE-baselen;
 		*from = c;
@@ -580,11 +596,12 @@ logit(file, id, from, to)
 		else
 			printf("%s: file: %s %s\n", id, file, to);
 #ifdef	LOG_INFO
-		if (flen > 0)
-			syslog(LOG_INFO, "%s: file: %s %s =>%s",
-					 id, file, from, to);
-		else
-			syslog(LOG_INFO, "%s: file: %s %s", id, file, to);
+		if (!nosyslog)
+		  if (flen > 0)
+		    syslog(LOG_INFO, "%s: file: %s %s =>%s",
+			   id, file, from, to);
+		  else
+		    syslog(LOG_INFO, "%s: file: %s %s", id, file, to);
 #endif	/* LOG_INFO */
 		to += MAXSAFESIZE-baselen-flen;
 		*to = c;
@@ -596,7 +613,8 @@ logit(file, id, from, to)
 	if (flen > 0) {
 		printf("%s: file: %s %s =>%s\n", id, file, from, to);
 #ifdef	LOG_INFO
-		syslog(LOG_INFO, "%s: file: %s %s =>%s", id, file, from, to);
+		if (!nosyslog)
+		  syslog(LOG_INFO, "%s: file: %s %s =>%s", id, file, from, to);
 #endif	/* LOG_INFO */
 	}
 }
