@@ -139,8 +139,7 @@ static int qpctlfile(spl)
 	    ++i;
 	  } else {
 	    if (vp->proc) {
-	      fprintf(qpfp, "(running now");
-	      fprintf(qpfp," pid=%d ",vp->proc->pid);
+	      fprintf(qpfp,"(running now, pid=%d ",vp->proc->pid);
 	      if (vp->proc->vertex == vp)
 		fprintf(qpfp,"active");
 	      else
@@ -157,7 +156,7 @@ static int qpctlfile(spl)
 		else
 		  fprintf(qpfp,"bygone");
 	      } else {
-		fprintf(qpfp,"fully inactive");
+		fprintf(qpfp," inactive");
 	      }
 	    }
 	    ++i;
