@@ -67,6 +67,7 @@ int	verbose = 0;
 int	querysocket = -1;	/* fd of TCP socket to listen for queries */
 int	D_alloc = 0;
 int	hungry_childs = 0;
+int	global_wrkcnt = 0;
 static int vtxprep_skip      = 0;
 static int vtxprep_skip_any  = 0;
 static int vtxprep_skip_lock = 0;
@@ -109,7 +110,7 @@ extern time_t time();
 extern u_char *rfc822date __((time_t *));
 extern void detach __((void));
 extern void die __((int rc, char *reason));
-extern void killprevious __((int signal, char *pidfile));
+extern int  killprevious __((int signal, char *pidfile));
 extern void prversion __((char *name));
 
 struct dirstatname {
@@ -147,7 +148,7 @@ struct ctlfile *cfp;
 	if (cfp->head != NULL) {
 	  for (vp = cfp->head; vp != NULL; vp = nvp) {
 	    nvp = vp->next[L_CTLFILE];
-	    unvertex(vp,1); /* Don't unlink! Just free()! */
+	    unvertex(vp,1,1); /* Don't unlink! Just free()! */
 	  }
 	} else {
 	  free(cfp->contents);
@@ -342,7 +343,11 @@ main(argc, argv)
 	}
 	/* Actually we want this to act as daemon,
 	   even when not in daemon mode.. */
-	killprevious(SIGTERM, pidfile);
+	if (killprevious(SIGTERM, pidfile) != 0) {
+	  printf("%s: Can't write my pid to a file ?? Out of diskspace ??\n",progname);
+	  die(1,"Can't write scheduler pid to a file!?");
+	  /* NOTREACHED */
+	}
 
 	for (i = 0; i < SIZE_L; ++i)
 	  spt_mesh[i] = sp_init();
@@ -682,6 +687,7 @@ static int syncweb(dq, cehead)
 	    /* Ok, schedule! */
 	    schedule(fd, file, cehead, ino);
 	    ++wrkcnt;
+	    ++global_wrkcnt;
 	  }
 	}
 
@@ -842,6 +848,7 @@ struct offsort {
 	long	headeroffset;
 	long	drptoffset;
 	char	*sender;
+	char	*dsnrecipient;
 };
 
 /* ``bcfcn'' is used by the qsort comparison routine,
@@ -982,8 +989,12 @@ static struct ctlfile *vtxprep(cfp)
 	      offarr[opcnt].headeroffset = -1;
 	      offarr[opcnt].drptoffset = -1;
 	      offarr[opcnt].sender = latest_sender;
+	      offarr[opcnt].dsnrecipient = NULL;
 	      ++opcnt;
 	      break;
+	    case _CF_DSNRECIPIENT:
+	      offarr[svn].dsnrecipient = cp+2;
+	      break;
 	    case _CF_RCPTNOTARY:
 	      /* XX: IETF-NOTARY-DRPT DATA! */
 	      offarr[svn].drptoffset = *lp + 2;
