@@ -95,6 +95,7 @@ struct threadgroup {
 struct thread {
 	time_t		wakeup;		/* When to wake up ?		    */
 	int		attempts;	/* How many times activated ?	    */
+	int		retryindex;	/* when, what ?			    */
 	char		*channel, *host; /* documenting */
 	struct thread	*nexttr;	/* Next one in threads queue	    */
 	struct thread	*prevtr;	/* previous one ...		    */
@@ -148,6 +149,7 @@ struct vertex {
 	long		headeroffset;	/* Message headers for this rcpt     */
 	long		drptoffset;	/* IETF-NOTARY DRPT  data	     */
 	char		*notary;	/* IETF Notary report data	     */
+	char		*dnsrecipient;	/*				     */
 
 	struct config_entry ce;		/* consed scheduler config file entry*/
 	struct procinfo *proc;		/* current TA process description    */
