@@ -47,7 +47,7 @@ int	numkids = 0;
 
 extern char *getzenv();
 extern char *strchr();
-extern int cistrcmp(), ranny();
+extern int cistrcmp();
 extern int errno;
 extern time_t time();
 
@@ -321,7 +321,10 @@ if (verbose)
 	  nvp = proc->thread->vertices;
 	  proc->thg->transporters -= 1;
 	  --numkids;
+	  if (proc->vertex != NULL)
+	    proc->vertex->proc = NULL;
 	  proc->vertex = NULL;
+	  proc->thread->proc = NULL;
 	  proc->thread = NULL;
 	  proc->thg    = NULL;
 	  /* XX: thread reschedule ?? */
