@@ -877,78 +877,3 @@ headers_need_mime2(rp)
 	}
 	return 0; /* No 8-bit chars in the headers */
 }
-
-
-/* This is rather half-baked approach, it creates correct tokens,
-   but if two tokens wind up adjacent, or one's length exceeds
-   recommended maximum, or ... then things aren't perfect. */
-
-int
-headers_to_mime2(rp,verboselog)
-	struct rcpt *rp;
-	FILE *verboselog;
-{
-	char **inhdr;
-
-	if (*(rp->newmsgheadercvt) == NULL)
-	  if (!cvtspace_copy(rp))
-	    return -1;	/* Failed to copy ! */
-
-	inhdr = *(rp->newmsgheadercvt);
-
-	while (inhdr && *inhdr) {
-	  char *hdr = *inhdr;
-	  u_char *s, *p, *q;
-	  int len;
-	  u_char *newbuf = NULL;
-
-      qphdr_restart:
-
-	  for (s = (u_char*)hdr; *s; ++s) {
-	    if (*s != '\t' && (*s < ' ' || *s > 126)) {
-
-	      /* Bad stuff, can't exist in the headers! */
-if (verboselog) fprintf(verboselog,"8-bit header: '%s'\n",hdr);
-
-	      /* Now rewind BACK to begin of this token,
-		 separators are: ' ','\t','(' */
-	      while (s > (u_char*)hdr && *s != ' ' &&
-		     *s != '\t' && *s != '(')
-		--s;
-	      if (*s == ' ' || *s == '\t') ++s;
-	      /* Now the 's' points at the begin of the token */
-	      p = (u_char*)hdr;
-	      if (!newbuf) {
-		len = strlen(s)*3; /* If it ALL turns into QP */
-		len += ((char*)s - hdr) + 30; /* Slag at the length */
-		newbuf = (u_char*)emalloc(len);
-	      }
-	      /* Copy the head */
-	      q = newbuf;
-	      while (p < s) *q++ = *p++;
-	      strcpy(q,"=?UNKNOWN-8BIT?Q?");
-	      q += 17; /* 17 chars above */
-
-	      for ( ; *s && (*s != ' ' && *s != '\t' && *s != ')'); ++s) {
-		if (*s < ' '  || *s > 126  ||
-		    *s == '=' || *s == '?' || *s == '_') {
-		  sprintf(q,"=%02X",*s);
-		  q += 3;
-		} else
-		  *q++ = *s;
-	      }
-	      strcpy(q,"?="); q += 2;
-	      strcpy(q,s);
-if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
-	      ctlfree(rp->desc,*inhdr);
-	      hdr = *inhdr = (char *)newbuf;
-	      newbuf = NULL;
-	      goto qphdr_restart;
-	    }
-	  }
-	  if (newbuf) free(newbuf);
-
-	  ++inhdr;
-	}
-	return 0;
-}
