@@ -253,6 +253,7 @@ extern int smtpwrite __((FILE *smtpfp[2], FILE *verboselog, int saverpt, char *l
 extern int process __((struct ctldesc*,int,char*,FILE**,FILE*));
 
 extern int check_7bit_cleanness __((struct ctldesc *dp));
+extern void notarystatsave __((char *smtpstatline, char *status));
 
 extern void prversion();
 extern void getnobody();
@@ -558,6 +559,7 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 {
 	struct rcpt *rp, *rphead;
 	int loggedid;
+	int openstatus = EX_OK;
 
 #ifndef	USE_MMAP
 	readalready = 0; /* ignore any previous message data cache */
@@ -568,7 +570,7 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 	  if (rp->next == NULL
 	      || rp->addr->link != rp->next->addr->link
 	      || rp->newmsgheader != rp->next->newmsgheader) {
-	    if (smtpstatus == EX_OK) {
+	    if (smtpstatus == EX_OK && openstatus == EX_OK) {
 	      if (logfp != NULL && !loggedid) {
 		loggedid = 1;
 		fprintf(logfp,
@@ -581,13 +583,13 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 		if ((first_uid = atoi(dp->senders->misc)) < 0 ||
 		    first_uid == nobody)
 		  first_uid = daemon_uid;
-		smtpstatus = smtpopen(host, smtpfp, verboselog);
-		if (smtpstatus == EX_OK)
+		openstatus = smtpopen(host, smtpfp, verboselog);
+		if (openstatus == EX_OK)
 		  smtp_isopen = 1;
 	      }
 	      smtpstatus = deliver(dp, smtpfp, verboselog, rphead, rp->next);
 	      /*
-		 if (smtpstatus != EX_OK) {
+		 if (openstatus != EX_OK) {
 		 fclose(smtpfp[0]); smtpfp[0] = NULL;
 		 fclose(smtpfp[1]); smtpfp[1] = NULL;
 		 if (logfp)
@@ -598,8 +600,22 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 	      rphead = rp->next;
 	    } else {
 	      while (rphead != rp->next) {
-		notaryreport(rp->addr->user,"failed",NULL);
-		diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
+		if (openstatus == EX_OK) {
+		  /* SMTP open -- meaning (propably) that we got reject
+		     from the remote server */
+		  /* NOTARY: addres / action / status / diagnostic */
+		  notaryreport(rp->addr->user,"failed",
+			       "5.1.1 (Target user does not exist)",
+			       NULL);
+		  diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
+		} else {
+		  /* SMTP not open -- remote didn't respond */
+		  /* NOTARY: addres / action / status / diagnostic */
+		  notaryreport(rp->addr->user,"failed",
+			       "5.4.1 (No answer from remote)",
+			       NULL);
+		  diagnostic(rphead, smtpstatus, 60, "%s", remotemsg);
+		}
 		rphead = rphead->next;
 	      }
 	    }
@@ -716,7 +732,9 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf);
 	if (r != EX_OK) {
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
-	    notaryreport(rp->addr->user,notaryacct(r,"*NOTOK*"),NULL);
+	    /* NOTARY: addres / action / status / diagnostic */
+	    notaryreport(rp->addr->user,notaryacct(r,"*NOTOK*"),
+			 "4.5.0 (Undetermined protocol error)",NULL);
 	    diagnostic(rp, r, 0, "%s", remotemsg);
 	  }
 	  if (smtp_isopen)
@@ -749,10 +767,12 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    }
 	  }
 	  
-	  notaryreport(rp->addr->user, NULL, NULL);
+	  /* NOTARY: addres / action / status / diagnostic */
+	  notaryreport(rp->addr->user, NULL, NULL, NULL);
 	  r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf);
 	  if (r != EX_OK) {
-	    notaryreport(NULL, notaryacct(r,"*NOTOK"),NULL);
+	    /* NOTARY: addres / action / status / diagnostic / wtt */
+	    notaryreport(NULL, notaryacct(r,"*NOTOK"),NULL,NULL);
 	    diagnostic(rp, r, 0, "%s", remotemsg);
 	    if (!smtp_isopen)
 	      break;
@@ -773,7 +793,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	if ((r = smtpwrite(smtpfp, verboselog, 1, "DATA")) != EX_OK) {
 	  for (rp = startrp; rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
-	      notaryreport(rp->addr->user,notaryacct(r,"*NOTOK"),NULL);
+	      /* NOTARY: addres / action / status / diagnostic / wtt */
+	      notaryreport(rp->addr->user,notaryacct(r,"*NOTOK"),NULL,NULL);
 	      diagnostic(rp, EX_TEMPFAIL, 0, "%s", remotemsg);
 	    }
 	  if (smtp_isopen)
@@ -799,8 +820,10 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 				  convertmode, 0)) < 0) {
 	  for (rp = startrp; rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
+	      /* NOTARY: addres / action / status / diagnostic / wtt */
 	      notaryreport(rp->addr->user,notaryacct(EX_TEMPFAIL,"*NOTOK*"),
-			   "466 (Message write timed out)"); /* XX: FIX THE STATUS? */
+			   "4.4.2 (Message write timed out)",
+			   "smtp; 466 (Message write timed out)"); /* XX: FIX THE STATUS? */
 	      diagnostic(rp, EX_TEMPFAIL, 0, "%s", "header write error");
 	    }
 	  if (verboselog)
@@ -822,7 +845,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  for (rp = startrp; rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user, notaryacct(r,"*NOTOK*"),
-			   "466 (Message write timed out)"); /* XX: FIX THE STATUS? */
+			   "4.4.2 (Message write timed out)",
+			   "smtp; 466 (Message write timed out)"); /* XX: FIX THE STATUS? */
 	      diagnostic(rp, r, 0, "%s", remotemsg);
 	    }
 	  /* The failure occurred during processing and was due to an I/O
@@ -864,7 +888,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  if (rp->status == EX_OK) {
 	    notaryreport(rp->addr->user,
 			 notaryacct(r,"relayed"), /* XXX: NOTARY DRPT ?? */
-			 NULL);
+			 NULL,NULL);
 	    diagnostic(rp, r, 0, "%s", remotemsg);
 	  }
 	if (r != EX_OK && smtp_isopen)
@@ -1110,14 +1134,16 @@ writebuf(fp, buf, len)
 	    state = 0;
 	    if (c == '.') {
 	      if (putc(c, fp) == EOF || putc(c, fp) == EOF) {
-		notaryreport(NULL,NULL,"500 (body write error, 1)");
+		notaryreport(NULL,NULL,"4.4.2 (body write error, 1)",
+			     "smtp; 500 (body write error, 1)");
 		strcpy(remotemsg, "write error 1");
 		return EOF;
 	      }
 	      VLFPRINTF((verboselog,".."));
 	    } else {
 	      if (putc(c, fp) == EOF) {
-		notaryreport(NULL,NULL,"500 (body write error, 2)");
+		notaryreport(NULL,NULL,"4.4.2 (body write error, 2)",
+			     "smtp; 500 (body write error, 2)");
 		strcpy(remotemsg, "write error 2");
 		return EOF;
 	      }
@@ -1125,7 +1151,8 @@ writebuf(fp, buf, len)
 	    }
 	  } else if (c == '\n') {
 	    if (putc('\r', fp) == EOF || putc(c, fp) == EOF) {
-	      notaryreport(NULL,NULL,"500 (body write error, 3)");
+	      notaryreport(NULL,NULL,"4.4.2 (body write error, 3)",
+			   "smtp; 500 (body write error, 3)");
 	      strcpy(remotemsg, "write error 3");
 	      return EOF;
 	    }
@@ -1133,7 +1160,8 @@ writebuf(fp, buf, len)
 	    state = 1;
 	  } else {
 	    if (putc(c, fp) == EOF) {
-	      notaryreport(NULL,NULL,"500 (body write error, 4)");
+	      notaryreport(NULL,NULL,"4.4.2 (body write error, 4)",
+			   "smtp; 500 (body write error, 4)");
 	      strcpy(remotemsg, "write error 4");
 	      return EOF;
 	    }
@@ -1259,8 +1287,9 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 
 	  if (column == 0 && c == '.') {
 	    if (putc(c, fp) == EOF) {
-	      notaryreport(NULL,NULL,"500 (body write error, 1)");
-	      strcpy(remotemsg, "write error 1");
+	      notaryreport(NULL,NULL,"4.4.2 (body write error, 5)",
+			   "smtp; 500 (body write error, 5)");
+	      strcpy(remotemsg, "write error 5");
 	      return EOF;
 	    }
 	    VLFPRINTF((verboselog,".."));
@@ -1268,16 +1297,18 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 
 	  if (c == '\n') {
 	    if (putc('\r', fp) == EOF) {
-	      notaryreport(NULL,NULL,"500 (body write error, 3)");
-	      strcpy(remotemsg, "write error 3");
+	      notaryreport(NULL,NULL,"4.4.2 (body write error, 6)",
+			   "smtp; 500 (body write error, 6)");
+	      strcpy(remotemsg, "write error 6");
 	      return EOF;
 	    }
 	    VLFPRINTF((verboselog,"\r"));
 	    column = -1;
 	  }
 	  if (putc(c, fp) == EOF) {
-	    notaryreport(NULL,NULL,"500 (body write error, 4)");
-	    strcpy(remotemsg, "write error 4");
+	      notaryreport(NULL,NULL,"4.4.2 (body write error, 7)",
+			   "smtp; 500 (body write error, 7)");
+	    strcpy(remotemsg, "write error 7");
 	    return EOF;
 	  }
 	  *lastcharp = c;
@@ -1434,10 +1465,11 @@ smtpconn(host, fparr, verboselog)
 	  *cp = '\0';
 	  ipaddr.s_addr = inet_addr(buf);
 	  if (ipaddr.s_addr == 0xFFFFFFFF) {
-	    sprintf(remotemsg, "500 (bad IP address: %s)", host);
-	    notaryreport(NULL,NULL,remotemsg);
+	    sprintf(remotemsg, "smtp; 500 (bad IP address: %s)", host);
+	    notaryreport(NULL,NULL,"5.1.2 (bad literal IP address)",
+			 remotemsg);
 	    if (verboselog)
-	      fprintf(verboselog,"%s\n", remotemsg);
+	      fprintf(verboselog,"%s\n", remotemsg+6);
 	    return EX_NOHOST;
 	  }
 	  hp = &hent;
@@ -1450,6 +1482,11 @@ smtpconn(host, fparr, verboselog)
 	  addrs[0] = (void *)&ipaddr;
 	  addrs[1] = NULL;
 	  hp_setalist(hp, addrs);
+	  {
+	    char nbuf[512];
+	    sprintf(nbuf,"X-IP-addr; [%s]", buf);
+	    notary_setwtt(nbuf);
+	  }
 	  retval = makeconn(hp, &smtpfd, verboselog, fparr, mxcnt);
 	} else {
 	  hbuf[0] = '\0';
@@ -1464,10 +1501,10 @@ smtpconn(host, fparr, verboselog)
 		|| h_errno == TRY_AGAIN
 #endif /* BIND */
 		) {
-	      sprintf(remotemsg,"466 (gethostbyname(%s): try later)",host);
-	      notaryreport(NULL,NULL,remotemsg);
+	      sprintf(remotemsg,"smtp; 466 (gethostbyname(%s): try later)",host);
+	      notaryreport(NULL,NULL,"4.4.3 (dns lookup timeout)",remotemsg);
 	      if (verboselog)
-		fprintf(verboselog,"%s\n",remotemsg);
+		fprintf(verboselog,"%s\n",remotemsg+6);
 	      return EX_TEMPFAIL;
 	    }
 #ifdef	BIND
@@ -1479,19 +1516,19 @@ smtpconn(host, fparr, verboselog)
 	    if ((i = getrr(hbuf, sizeof hbuf, (u_short)T_CNAME,2)) == 1) {
 	      host = hbuf;
 	    } else if (h_errno == TRY_AGAIN) {
-	      sprintf(remotemsg, "466 (Canonicalize, try later: %s)", host);
-	      notaryreport(NULL,NULL,remotemsg);
+	      sprintf(remotemsg, "smtp; 466 (Canonicalize, try later: %s)", host);
+	      notaryreport(NULL,NULL,"4.4.3 (dns lookup 'try again')",remotemsg);
 	      if (verboselog)
-		fprintf(verboselog,"%s\n",remotemsg);
+		fprintf(verboselog,"%s\n",remotemsg+6);
 	      return EX_TEMPFAIL;
 	    } /* else [Matti Aarnio says:]
 		 it is possible that MXs or As are available
 		 even though CNAME lookups fail (see .UK) */
 #else  /* !BIND */
-	    sprintf(remotemsg, "466 (canonicalize lookup failed on %s)", host);
-	    notaryreport(NULL,NULL,remotemsg);
+	    sprintf(remotemsg, "smtp; 466 (object lookup failed on %s)", host);
+	    notaryreport(NULL,NULL,"4.4.3 (unspecified dns error)", remotemsg);
 	    if (verboselog)
-	      fprintf(verboselog,"%s\n", remotemsg);
+	      fprintf(verboselog,"%s\n", remotemsg+6);
 	    /* temporary failure since the router knew it */
 	    return EX_TEMPFAIL;	/* was EX_NOHOST */
 #endif /* BIND */
@@ -1509,7 +1546,7 @@ smtpconn(host, fparr, verboselog)
 	    int rc;
 	    if (verboselog)
 	      fprintf(verboselog," getmxrr(%s)",host);
-	    rc = getmxrr(host, mxh, sizeof mxh/sizeof mxh[0],0,&mxcnt);
+	    rc = getmxrr(host, mxh, MAXFORWARDERS, 0, &mxcnt);
 	    if (verboselog)
 	      if (mxcnt == 0)
 		fprintf(verboselog, " rc=%d, no MXes (host=%s)\n", rc, host);
@@ -1539,28 +1576,35 @@ smtpconn(host, fparr, verboselog)
 		  || h_errno == TRY_AGAIN
 #endif /* BIND */
 		  ) {
-		sprintf(remotemsg,"466 (gethostbyname<%s>: try later)",host);
-		notaryreport(NULL,NULL,remotemsg);
+		sprintf(remotemsg,"smtp; 466 (gethostbyname<%s>: try later)",host);
+		notaryreport(NULL,NULL,"4.4.3 (dns lookup 'try again')", remotemsg);
 		if (verboselog)
-		  fprintf(verboselog,"%s\n",remotemsg);
+		  fprintf(verboselog,"%s\n",remotemsg+6);
 		return EX_TEMPFAIL;
 	      }
 
 	      if (noMX)
 		sprintf(remotemsg,
-			"500 (configuration inconsistency. MX usage forbidden, no address in DNS: '%s', errno=%s)",
+			"smtp; 500 (configuration inconsistency. MX usage forbidden, no address in DNS: '%s', errno=%s)",
 			host,strerror(errno));
 	      else
 		sprintf(remotemsg,
-			"500 (nameserver data inconsistency. No MX, no address: '%s', errno=%s)",
+			"smtp; 500 (nameserver data inconsistency. No MX, no address: '%s', errno=%s)",
 			host,strerror(errno));
-	      notaryreport(NULL,NULL,remotemsg);
+	      notaryreport(NULL,NULL,"4.4.4 (nameserver data inconsistency)",
+			   remotemsg);
 	      if (verboselog)
-		fprintf(verboselog,"%s\n",remotemsg);
+		fprintf(verboselog,"%s\n",remotemsg+6);
 	      /* it was: EX_UNAVAILABLE, but such blocks retrying, thus
 		 current EX_TEMPFAIL, which will cause timeout latter on.. */
 	      return EX_TEMPFAIL;
 	    }
+	    hp_addr_randomize(hp);
+	    {
+	      char buf[512];
+	      sprintf(buf,"dns; %s", host);
+	      notary_setwtt(buf);
+	    }
 	    retval = makeconn(hp, &smtpfd, verboselog, fparr, mxcnt);
 	  } else {
 	    retval = EX_UNAVAILABLE;
@@ -1571,6 +1615,12 @@ smtpconn(host, fparr, verboselog)
 		firstmx = i+1;
 		continue;
 	      }
+	      hp_addr_randomize(hp);
+	      {
+		char buf[512];
+		sprintf(buf,"dns; %s", mxh[i].host);
+		notary_setwtt(buf);
+	      }
 	      r = makeconn(hp, &smtpfd, verboselog, fparr, mxcnt);
 	      if (r == EX_OK) {
 		while (mxh[i].host != NULL) {
@@ -1620,20 +1670,30 @@ makeconn(hp, fdp, verboselog, fparr, mxcnt)
 	strcpy(hostbuf, hp->h_name);
 	if (checkwks &&
 	    getrr(hostbuf, sizeof hostbuf, (u_short)T_WKS, 2) != 1) {
-	  sprintf(remotemsg,"550 (WKS checks: no SMTP capability for host %s)",
+	  sprintf(remotemsg,"smtp; 550 (WKS checks: no SMTP capability for host %s)",
 		  hostbuf);
-	  notaryreport(NULL,NULL,remotemsg);
+	  notaryreport(NULL,NULL,"4.4.4 (WKS Checks: no SMTP capability)", remotemsg);
 	  if (verboselog)
-	    fprintf(verboselog,"%s\n",remotemsg);
+	    fprintf(verboselog,"%s\n",remotemsg+6);
 	  return EX_UNAVAILABLE;
 	}
 #endif	/* RFC974 */
 #endif	/* BIND */
 
 	retval = EX_UNAVAILABLE;
+#if 0
+	if (verboselog) {
+	  fprintf(verboselog,"makeconn('%s') to IP addresses:", hostbuf);
+	  for (hp_init(hp); *hp_getaddr() != NULL; hp_nextaddr()) {
+	    fprintf(verboselog," %s",
+		    dottedquad((struct in_addr*)*hp_getaddr()));
+	  }
+	  fprintf(verboselog,"\n");
+	}
+#endif
 	for (hp_init(hp); *hp_getaddr() != NULL; hp_nextaddr()) {
 	  if (matchmyaddress(*hp_getaddr(),hp->h_length)) {
-	    notaryreport(NULL,NULL,"500 (Trying to talk with myself!)");
+	    notaryreport(NULL,NULL,"5.4.6 (trying to talk with myself!)","smtp; 500 (Trying to talk with myself!)");
 	    sprintf(remotemsg,"Trying to talk with myself!");
 	    break;		/* TEMPFAIL or UNAVAILABLE.. */
 	  }
@@ -1713,12 +1773,12 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	sad.sin_addr.s_addr = 0;
 	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	  se = strerror(errno);
-	  sprintf(remotemsg, "500 (Internal error, socket: %s)", se);
-	  notaryreport(NULL,NULL,remotemsg);
+	  sprintf(remotemsg, "smtp; 500 (Internal error, socket: %s)", se);
+	  notaryreport(NULL,NULL,"4.4.0 (internal error)",remotemsg);
 	  if (conndebug)
-	    fprintf(stderr, "%s\n", remotemsg);
+	    fprintf(stderr, "%s\n", remotemsg+6);
 	  if (verboselog)
-	    fprintf(verboselog,"%s\n",remotemsg);
+	    fprintf(verboselog,"%s\n",remotemsg+6);
 	  if (logfp)
 	    fprintf(logfp,"%d#\t(Internal error, socket: %s)\n",pid,se);
 	  return EX_TEMPFAIL;
@@ -1730,12 +1790,12 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	      break;
 	    if (errno != EADDRINUSE && errno != EADDRNOTAVAIL) {
 	      char *s = strerror(errno);
-	      sprintf(remotemsg, "500 (Internal error, bind: %s)", s);
-	      notaryreport(NULL,NULL,remotemsg);
+	      sprintf(remotemsg, "smtp; 500 (Internal error, bind: %s)", s);
+	      notaryreport(NULL,NULL,"4.4.0 (internal error, bind)",remotemsg);
 	      if (verboselog)
-		fprintf(verboselog,"%s\n", remotemsg);
+		fprintf(verboselog,"%s\n", remotemsg+6);
 	      if (conndebug)
-		fprintf(stderr, "%s\n", remotemsg);
+		fprintf(stderr, "%s\n", remotemsg+6);
 	      if (logfp)
 		fprintf(logfp, "%d#\t(Internal error, bind: %s)\n", pid, s);
 	      return EX_UNAVAILABLE;
@@ -1743,11 +1803,11 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	  }
 	  if (p < IPPORT_RESERVED/2) {
 	    sprintf(remotemsg, "too many busy ports");
-	    notaryreport(NULL,NULL,"500 (Internal error, too many busy ports)");
+	    notaryreport(NULL,NULL,"4.4.0 (internal error, too many busy ports)","smtp; 500 (Internal error, too many busy ports)");
 	    if (conndebug)
-	      fprintf(stderr, "%s\n", remotemsg);
+	      fprintf(stderr, "%s\n", remotemsg+6);
 	    if (verboselog)
-	      fprintf(verboselog,"%s\n",remotemsg);
+	      fprintf(verboselog,"%s\n",remotemsg+6);
 	    if (logfp)
 	      fprintf(logfp,"%d#\t(Internal error, too many busy ports)\n", pid);
 	    return EX_TEMPFAIL;
@@ -1774,11 +1834,11 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 
 	errnosave = errno;
 	se = strerror(errno);
-	sprintf(remotemsg, "500 (connect to %s [%s]: %s)",
+	sprintf(remotemsg, "smtp; 500 (connect to %s [%s]: %s)",
 		hostname, ipaddress, se);
 	if (statusreport)
 	  report("%s",remotemsg+4);
-	notaryreport(NULL,NULL,remotemsg);
+	notaryreport(NULL,NULL,"4.4.1 (TCP/IP-connection failure)", remotemsg);
 
 	if (conndebug)
 		fprintf(stderr, "%s\n", remotemsg);
@@ -1866,8 +1926,10 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	char *strbuf;
 {
 	register char *s, *cp;
-	int i, response, r, infd, outfd;
+	int i, response, r, infd, outfd, rc;
 	char *se;
+	char *status = NULL;
+	char *diags  = NULL;
 	char buf[8192]; /* XX: Eh... Static buffer.. */
 	char ch;
 
@@ -1891,12 +1953,13 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  if ((r = write(outfd, buf, len)) == -1) {
 	    if (gotalarm) {
 	      strcpy(remotemsg, "Timeout on cmd write");
-	      notaryreport(NULL,NULL,"500 (timeout on cmd write)");
+	      notaryreport(NULL,NULL,"4.4.2 (timeout on cmd write)",
+			   "smtp; 500 (timeout on cmd write)");
 	    } else
 	      {
 		se = strerror(errno);
-		sprintf(remotemsg, "500 (write to server error: %s)", se);
-		notaryreport(NULL,NULL,remotemsg);
+		sprintf(remotemsg, "smtp; 500 (write to server error: %s)", se);
+		notaryreport(NULL,NULL,"4.4.2 (write to server, err)",remotemsg);
 	      }
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
@@ -1907,8 +1970,8 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	      fprintf(logfp, "%d#\t(closed SMTP channel - timeout on smtpwrite())\n", pid);
 	    return EX_TEMPFAIL;
 	  } else if (r != len) {
-	    sprintf(remotemsg, "500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
-	    notaryreport(NULL,NULL,remotemsg);
+	    sprintf(remotemsg, "smtp; 500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
+	    notaryreport(NULL,NULL,"4.4.2 (SMTP cmd partial write failure)",remotemsg);
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
@@ -2010,24 +2073,23 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    if (gotalarm) {
 	      if (strbuf == NULL)
 		sprintf(remotemsg,
-			"466 (Timeout on initial SMTP responce read)");
+			"smtp; 466 (Timeout on initial SMTP responce read)");
 	      else
 		sprintf(remotemsg,
-			"466 (Timeout on SMTP responce read, Cmd: %s)",
+			"smtp; 466 (Timeout on SMTP responce read, Cmd: %s)",
 			strbuf);
-	      notaryreport(NULL,NULL,remotemsg);
-	    } else
-	      {
+	      notaryreport(NULL,NULL,"4.4.2 (smtp transaction read timeout)",remotemsg);
+	    } else {
 		se = strerror(errno);
 		if (strbuf == NULL)
 		  sprintf(remotemsg,
-			  "500 (Error on initial SMTP responce read: %s)",se);
+			  "smtp; 500 (Error on initial SMTP responce read: %s)",se);
 
 		else
 		  sprintf(remotemsg,
-			  "500 (Error on SMTP responce read: %s, Cmd: %s)",
+			  "smtp; 500 (Error on SMTP responce read: %s, Cmd: %s)",
 			  se, strbuf);
-		notaryreport(NULL,NULL,remotemsg);
+		notaryreport(NULL,NULL,"4.4.2 (smtp transaction read timeout)",remotemsg);
 	      }
 	    dflag = 0;
 	    firstmx = 0;
@@ -2040,9 +2102,9 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	      fprintf(logfp, "%d#\t(closed SMTP channel - bad responce on smtpwrite() )\n", pid);
 	    return EX_TEMPFAIL;
 	  } else {
-	    sprintf(remotemsg, "500 (Server hung up on us! Cmd: %s)",
+	    sprintf(remotemsg, "smtp; 500 (Server hung up on us! Cmd: %s)",
 		    strbuf == NULL ? "(null cmd)" : strbuf);
-	    notaryreport(NULL,NULL,remotemsg);
+	    notaryreport(NULL,NULL,"4.4.2 (server hung-up on us)",remotemsg);
 	    dflag = 0;
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
@@ -2059,8 +2121,8 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	} while (cp < buf+sizeof buf && !(i == 0 && *(cp-1) == '\n'));
 
 	if (cp >= buf+sizeof buf) {
-	  strcpy(remotemsg,"500 (SMTP Response overran input buffer!)");
-	  notaryreport(NULL,NULL,remotemsg);
+	  strcpy(remotemsg,"smtp; 500 (SMTP Response overran input buffer!)");
+	  notaryreport(NULL,NULL,"5.5.0 (SMTP-responce overran input buffer!)",remotemsg);
 	  dflag = 0;
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg);
@@ -2073,8 +2135,8 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	}
 	*--cp = '\0';	/* kill the LF */
 	if (cp - buf < 5) {
-	  sprintf(remotemsg, "500 (SMTP response '%s' unexpected!)", buf);
-	  notaryreport(NULL,NULL,remotemsg);
+	  sprintf(remotemsg, "smtp; 500 (SMTP response '%s' unexpected!)", buf);
+	  notaryreport(NULL,NULL,"5.5.0 (SMTP responce unexpected)",remotemsg);
 	  dflag = 0;
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg);
@@ -2109,8 +2171,6 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  esmtp_banner_check(&buf[4]);
 
 	rmsgappend("\r->> %s",buf);
-	if (saverpt)
-	  notarystatsave(buf);
 
 	
 	dflag = 0;
@@ -2123,42 +2183,82 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	case 251: /* User not local; will forward to <forward-path> */
         case 255: /* Something the PMDF 4.1 returns.. for EHLO */
 	case 354: /* Start mail input; end with <CRLF>.<CRLF> */
-		return EX_OK;
+		status = "2.0.0";
+		rc = EX_OK;
+		break;
 	case 421: /* <domain> Service not available, closing transmission channel */
 	case 450: /* Requested mail action not taken: mailbox unavailable */
 	case 451: /* Requested action aborted: local error in processing */
 	case 452: /* Requested action not taken: insufficient system storage */
-		return EX_TEMPFAIL;
-	case 500: /* Syntax error, command unrecognized */
+		status = "4.0.0";
+		rc = EX_TEMPFAIL;
+		break;
+	case 455: /* ESMTP parameter failure */
+		status = "5.5.4";
+		rc = EX_USAGE;
+		break;
 	case 501: /* Syntax error in parameters or arguments */
-		return EX_USAGE;
+		status = "5.5.2";
+		rc = EX_USAGE;
+		break;
+	case 500: /* Syntax error, command unrecognized */
 	case 502: /* Command not implemented */
-		return EX_PROTOCOL;
+		status = "5.5.1";
+		rc = EX_PROTOCOL;
+		break;
 	case 503: /* Bad sequence of commands */
-		return EX_TEMPFAIL;
+		status = "5.5.0";
+		rc = EX_TEMPFAIL;
+		break;
 	case 504: /* Command parameter not implemented */
-		return EX_PROTOCOL;
+		status = "5.5.4";
+		rc = EX_PROTOCOL;
+		break;
 	case 550: /* Requested action not taken: mailbox unavailable */
+		status = "5.1.1 (bad destination mailbox)";
+		rc = EX_NOUSER;
+		break;
 	case 551: /* User not local; please try <forward-path> */
-		return EX_NOUSER;
+		status = "5.1.6 (mailbox has moved)";
+		rc = EX_NOUSER;
+		break;
 	case 552: /* Requested mail action aborted: exceeded storage allocation */
-		return EX_UNAVAILABLE;
+		status = "5.2.3 (message length exceeds administrative limit)";
+		rc = EX_UNAVAILABLE;
+		break;
 	case 553: /* Requested action not taken: mailbox name not allowed */
-		return EX_NOUSER;
-	case 554: /* Transaction failed */
-		return EX_UNAVAILABLE;
+		status = "5.1.3 (bad destination mailbox address syntax)";
+		rc = EX_NOUSER;
+		break;
+	case 555: /* Unknown MAIL FROM:<>/RCPT TO:<> parameter */
+		status = "5.5.4 (invalid parameters)";
+		rc = EX_USAGE;
+		break;
+	default:
+		switch (response/100) {
+		case 2:
+		case 3:
+			status = "2.0.0 (generic ok)";
+			rc = EX_OK;
+			break;
+		case 4:
+			status = "4.0.0 (generic temporary failure)";
+			rc = EX_TEMPFAIL;
+			break;
+		case 5:
+			status = "5.0.0 (generic permanent failure)";
+			rc = EX_UNAVAILABLE;
+			break;
+		default:
+			status = "5.5.0 (generic protocol failure)";
+			rc = EX_TEMPFAIL;
+			break;
+		}
 	}
-	switch (response/100) {
-	case 2:
-	case 3:
-		return EX_OK;
-	case 4:
-		return EX_TEMPFAIL;
-	case 5:
-		return EX_UNAVAILABLE;
-	}
-	return EX_TEMPFAIL;
-}
+
+	if (saverpt)
+	  notarystatsave(buf,status);
+	return rc;}
 
 
 int
@@ -2176,7 +2276,7 @@ smtp_ehlo(smtpfp, verboselog, strbuf)
 
 /*
  * In theory, this should modify the command that ps shows for this process.
- * This is known to not be portable, hopefully it will break badly on systems
+ * This is known not to be portable, hopefully it will break badly on systems
  * where it doesn't work.
  */
 
@@ -2297,7 +2397,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 
 	if (depth > 3) {
 	  sprintf(remotemsg,"500 (DNS: Recursive CNAME on '%s')",host);
-	  notaryreport(NULL,NULL,remotemsg);
+	  notaryreport(NULL,NULL,"4.4.3 (Recursive DNS CNAME)",remotemsg);
 	  fprintf(stderr, "%s\n", remotemsg);
 	  return EX_NOHOST;
 	}
@@ -2308,15 +2408,15 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	if (qlen < 0) {
 	  fprintf(stderr, "res_mkquery failed\n");
 	  sprintf(remotemsg,
-		  "466 (Internal: res_mkquery failed on host: %s)",host);
-	  notaryreport(NULL,NULL,remotemsg);
+		  "smtp; 466 (Internal: res_mkquery failed on host: %s)",host);
+	  notaryreport(NULL,NULL,"4.4.3 (DNS-failure)",remotemsg);
 	  return EX_SOFTWARE;
 	}
 	n = res_send((void*)&qbuf, qlen, (void*)&answer, sizeof answer);
 	if (n < 0) {
 	  sprintf(remotemsg,
-		  "466 (Internal: res_send failed on host: %s)",host);
-	  notaryreport(NULL,NULL,remotemsg);
+		  "smtp; 466 (Internal: res_send failed on host: %s)",host);
+	  notaryreport(NULL,NULL,"4.4.3 (DNS-failure)",remotemsg);
 	  return EX_TEMPFAIL;
 	}
 	eom = (msgdata *)&answer + n;
@@ -2334,12 +2434,12 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	     * due to a bug; if that is the case by you,
 	     * change to return EX_TEMPFAIL iff hp->aa == 0.
 	     */
-	    sprintf(remotemsg, "500 (Bind: no such domain: %s)", host);
-	    notaryreport(NULL,NULL,remotemsg);
+	    sprintf(remotemsg, "smtp; 500 (Bind: no such domain: %s)", host);
+	    notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOHOST;
 	  case SERVFAIL:
-	    sprintf(remotemsg, "500 (Bind: server failure: %s)", host);
-	    notaryreport(NULL,NULL,remotemsg);
+	    sprintf(remotemsg, "smtp; 500 (Bind: server failure: %s)", host);
+	    notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
 	    return EX_TEMPFAIL;
 #ifdef OLDJEEVES
 	    /*
@@ -2359,12 +2459,12 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 #endif
 	  case NOTIMP:
 	  case REFUSED:
-	    sprintf(remotemsg, "500 (Bind: unsupported query: %s)", host);
-	    notaryreport(NULL,NULL,remotemsg);
+	    sprintf(remotemsg, "smtp; 500 (Bind: unsupported query: %s)", host);
+	    notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOPERM;
 	  }
-	  sprintf(remotemsg, "500 (Bind: unknown error, MX info unavailable: %s)", host);
-	  notaryreport(NULL,NULL,remotemsg);
+	  sprintf(remotemsg, "smtp; 500 (Bind: unknown error, MX info unavailable: %s)", host);
+	  notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
 	nmx = 0;
@@ -2414,16 +2514,16 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	    *mxcntp += 1;
 	    continue;		/* Well well.. spurious! */
 	  }
-	  if (cistrcmp(hent->h_name, myhostname) == 0)
-	    maxpref = mx[nmx].pref;
-	  if (matchmyaddresses(hent))
-	    maxpref = mx[nmx].pref;
+	  if (cistrcmp(hent->h_name, myhostname) == 0
+	      || matchmyaddresses(hent))
+	    if (maxpref < 0 || maxpref > mx[nmx].pref)
+	      maxpref = mx[nmx].pref;
 
 	  if ((mx[nmx].host = (msgdata *)emalloc(strlen((void*)buf)+1)) == NULL) {
 	    fprintf(stderr, "Out of virtual memory!\n");
 	    exit(EX_OSERR);
 	  }
-	  strcpy(mx[nmx].host, (void*)buf);
+	  strcpy((char*)mx[nmx].host, (char*)buf);
 	  /* [mea] Canonicalize this target & get A records.. */
 	  ++nmx;
 	}
@@ -2460,7 +2560,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	for (n = 0; checkwks && n < nmx; ++n) {
 	  if (mx[n].host == NULL)
 	    continue;
-	  strcpy(buf, mx[n].host);
+	  strcpy((char*)buf, (char*)mx[n].host);
 	  /* It is an MX, it CAN'T have CNAME ! */
 	  if (!getrrtype((void*)buf, sizeof buf, T_WKS, 0)) {
 	    free(mx[n].host);
@@ -2481,7 +2581,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	}
 	if (n == 0) {/* MX's exist, but their WKS's show no TCP smtp service */
 	  sprintf(remotemsg, "500 (Bind: MX host does not support SMTP: %s)", host);
-	  notaryreport(NULL,NULL,remotemsg);
+	  notaryreport(NULL,NULL,"4.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
 	nmx = n;
@@ -2495,6 +2595,37 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	    }
 	  }
 	}
+
+	/* Randomize the order of those of same preferrence [mea]
+	   This will do some sort of load-balancing on large sites
+	   which have multiple mail-servers at the same priority.  */
+	for (i = 0, maxpref = mx[0].pref; i < nmx; ++i) {
+	  /* They are in numerical order, now we can
+	     detect when a new preferrence group steps in */
+	  j = i;
+	  while (j < nmx && maxpref == mx[j].pref) ++j;
+	  if ((j-i) > 1) {
+	    /* At least two of the same preferrence */
+	    int k, len = j-i;
+	    for (k = 0; k < len; ++k) {
+	      int l = ranny(len);
+	      mxtemp = mx[i+k];
+	      mx[i+k] = mx[i+l];
+	      mx[i+l] = mxtemp;
+	    }
+	  }
+	  /* Processed that preference, now next */
+	  i = j-1;
+	  if (j < nmx)		/* If within the array */
+	    maxpref = mx[j].pref;
+	}
+#if 0
+	if (verboselog) {
+	  fprintf(verboselog,"Target has following MXes (cnt=%d):\n",nmx);
+	  for (i=0; i<nmx; ++i)
+	    fprintf(verboselog,"  MX %3d %s\n", mx[i].pref, mx[i].host);
+	}
+#endif
 	mx[nmx].host = NULL;
 	mxcount = nmx;
 	return EX_OK;
@@ -2525,7 +2656,7 @@ rightmx(spec_host, addr_host)
 
 	mxh[0].host = NULL;
 	mxcnt = 0;
-	switch (getmxrr(addr_host, mxh, sizeof mxh/sizeof mxh[0], 0, &mxcnt)) {
+	switch (getmxrr(addr_host, mxh, MAXFORWARDERS, 0, &mxcnt)) {
 	case EX_OK:
 	  if (mxh[0].host == NULL)
 	    return cistrcmp(addr_host, remotehost) == 0;
@@ -2630,10 +2761,10 @@ struct ctldesc *dp;
 }
 
 void
-notarystatsave(smtpline)
-char *smtpline;
+notarystatsave(smtpline,status)
+char *smtpline, *status;
 {
-	int len = strlen(smtpline)+8;
+	int len = strlen(smtpline)+8+6;
 #ifdef USE_ALLOCA
 	char *str = alloca(len);
 #else
@@ -2641,12 +2772,15 @@ char *smtpline;
 #endif
 	char *s = str;
 
+	strcpy(s,"smtp; ");
+	s += 6;
+
 	*s++ = *smtpline++;
 	*s++ = *smtpline++;
 	*s++ = *smtpline++;
 	*s++ = ' ';
 	if (len < 11) {
-	  notaryreport(NULL,NULL,str);
+	  notaryreport(NULL,NULL,status,str);
 #ifndef USE_ALLOCA
 	  free(str);
 #endif
@@ -2668,7 +2802,7 @@ char *smtpline;
 	}
 	*s++ = ')';
 	*s = 0;
-	notaryreport(NULL,NULL,str);
+	notaryreport(NULL,NULL,status,str);
 #ifndef USE_ALLOCA
 	free(str);
 #endif
